@@ -39,6 +39,10 @@
  *
  */
 
+/**
+ * @file
+ */
+
 #ifndef OLSRV2_ROUTING_H_
 #define OLSRV2_ROUTING_H_
 
@@ -53,7 +57,8 @@
 #include "nhdp/nhdp_db.h"
 #include "nhdp/nhdp_domain.h"
 
-#define OLSRv2_DIJKSTRA_RATE_LIMITATION 1000
+/*! minimum time between two dijkstra calculations in milliseconds */
+enum { OLSRv2_DIJKSTRA_RATE_LIMITATION = 1000 };
 
 /**
  * representation of a node in the dijkstra tree
@@ -71,6 +76,9 @@ struct olsrv2_dijkstra_node {
   /*! hopcount to be inserted into the route */
   uint8_t distance;
 
+  /*! originator address of the router responsible for the prefix */
+  const struct netaddr *originator;
+
   /*! pointer to nhpd neighbor that represents the first hop */
   struct nhdp_neighbor *first_hop;
 
@@ -106,6 +114,9 @@ struct olsrv2_routing_entry {
   /*! path hops to the target */
   uint8_t path_hops;
 
+  /*! originator of node that announced the route */
+  struct netaddr originator;
+
   /*! originator address of next hop */
   struct netaddr next_originator;
 
@@ -121,14 +132,8 @@ struct olsrv2_routing_entry {
   /*! true if this route is being processed by the kernel at the moment */
   bool in_processing;
 
-  /*! interface index before the current dijkstra run */
-  unsigned _old_if_index;
-
-  /*! next hop before the current dijkstra run */
-  struct netaddr _old_next_hop;
-
-  /*! distance before the current dijkstra run */
-  uint8_t _old_distance;
+  /*! old values of route before current dijstra run */
+  struct os_route_parameter _old;
 
   /*! hook into working queues */
   struct list_entity _working_node;
@@ -162,13 +167,13 @@ struct olsrv2_routing_domain {
  */
 struct olsrv2_routing_filter {
   /**
-   * callback for routing filter, return false to drop route.
-   * filter can modify rt_table, rt_protocol and rt_metric.
+   * callback for routing filter, return false to drop route modification.
+   * filter can modify all parameters of the route.
    * @param domain NHDP domain
-   * @param route operation system route
-   * @return true if route should still be set, false if it should be dropped
+   * @param route_param operation system route parameters
+   * @return true if route should still be set/removed, false if it should be ignored
    */
-  bool (*filter)(struct nhdp_domain *domain , struct os_route *route);
+  bool (*filter)(struct nhdp_domain *domain , struct os_route_parameter *route_param, bool set);
 
   /*! node to hold all routing filters together */
   struct list_entity _node;
@@ -178,7 +183,8 @@ void olsrv2_routing_init(void);
 void olsrv2_routing_initiate_shutdown(void);
 void olsrv2_routing_cleanup(void);
 
-void olsrv2_routing_dijkstra_node_init(struct olsrv2_dijkstra_node *);
+void olsrv2_routing_dijkstra_node_init(
+    struct olsrv2_dijkstra_node *, const struct netaddr *originator);
 
 EXPORT void olsrv2_routing_set_domain_parameter(struct nhdp_domain *domain,
     struct olsrv2_routing_domain *parameter);
@@ -186,6 +192,8 @@ EXPORT void olsrv2_routing_set_domain_parameter(struct nhdp_domain *domain,
 EXPORT void olsrv2_routing_force_update(bool skip_wait);
 EXPORT void olsrv2_routing_trigger_update(void);
 
+EXPORT void olsrv2_routing_freeze_routes(bool freeze);
+
 EXPORT const struct olsrv2_routing_domain *
     olsrv2_routing_get_parameters(struct nhdp_domain *);
 
